<?php
/*
Plugin Name: ExportYourStore
description: Easily integrate your WooCommerce store with the largest online marketplaces.
Version: 1.0
Author: ExportYourStore
Author URI: https://www.exportyourstore.com/
Text Domain : eys
License: GPL2
 */
if (!defined('ABSPATH')) {
    exit;
}

class eys
{
    /**
     *
     *
     * @since 0.1.0
     * @var string
     */
    public $version = '1.0';

    /*
     * construct function
     *
     * @since 0.1.0
     */
    public function __construct()
    {
        add_action('admin_menu', array($this, 'eys_my_admin_menu'));
        add_action('admin_enqueue_scripts', array($this, 'eys_styles_enque'));
        add_action('rest_api_init', array($this,'create_rest_api_redirect_eys_user'));
        add_action('init', array($this, 'eys_rewrites_init'));
        add_filter('query_vars', array($this,'eys_query_vars'));
        //Load template from specific page
        add_filter('template_include', array($this, 'eys_page_template' ));
        if ($this->eys_is_wc_active()) {
        }
    }
    
    /**
     * eys_style_enque
     *
     * @return void
     */
    public function eys_styles_enque()
    {
        wp_register_style('eys_style_css', plugins_url('/css/eys_style.css', __FILE__));
        wp_enqueue_style('eys_style_css');
    }

    /**
     * Include plugin file.
     *
     * @since 0.1.0
     *
     */
    public function includes()
    {
        // require_once $this->get_plugin_path() . '/includes/woo-tab.php';
    }
    
    /**
     * eys_my_admin_menu
     *
     * @return void
     */
    public function eys_my_admin_menu()
    {
        add_menu_page(
		__('ExportYourStore', 'eys'), 
		__('ExportYourStore', 'eys'), 
		'manage_options', 'eys-setting-page', 
		array($this, 'eys_setting_page')
		);
    }


  
    public function eys_rewrites_init()
    {
        add_rewrite_rule('^exportyourstore/([a-z0-9-]+)[/]?$', 'index.php?user_id=$matches[1]', 'top');
    }
    public function eys_query_vars($query_vars)
    {
        $query_vars[] = 'user_id';
        return $query_vars;
    }


   
    public function eys_page_template($page_template)
    {
        if (get_query_var('user_id') == false || get_query_var('user_id') == '') {
            return $page_template;
        }
       
        return  dirname(__FILE__) . '/templates/page-ExportYourStore.php';
    }
        
    /**
     * eys_setting_p
     *
     * @return void
     */
    public function eys_setting_page()
    {
        global $wpdb;
        $consumer_key ='';
        $consumer_secret = '';
        $baseUrl= $ExportYourStore_url = 'https://app.exportyourstore.com/';
        $user_id = get_current_user_id();
        $keys = $wpdb->get_row($wpdb->prepare("
			SELECT consumer_key, consumer_secret, permissions
			FROM {$wpdb->prefix}woocommerce_api_keys
			WHERE user_id = %d
		", $user_id), ARRAY_A);
        if (get_option('eys_woocommerce_api_consumer_key') && get_option('eys_woocommerce_api_consumer_secret')) {
            $consumer_key = get_option('eys_woocommerce_api_consumer_key');
            $consumer_secret = get_option('eys_woocommerce_api_consumer_secret');
            $query ="signup?wc=1&ck=".  $consumer_key."&cs=". $consumer_secret."&url=".get_site_url();
            $ExportYourStore_url =  $ExportYourStore_url.$query;
        } else {
            $_POST['key_id'] =0;
            $_POST['user'] = get_current_user_id();
            $_POST['description'] ='ExportYourStore';
            $_POST['permissions'] ='read_write';
    
            if (! current_user_can('manage_woocommerce')) {
                wp_die(-1);
            }
    
            $response = array();
    
            try {
                if (empty($_POST['description'])) {
                    throw new Exception(__('Description is missing.', 'eys'));
                }
                if (empty($_POST['user'])) {
                    throw new Exception(__('User is missing.', 'eys'));
                }
                if (empty($_POST['permissions'])) {
                    throw new Exception(__('Permissions is missing.', 'eys'));
                }
    
                $key_id      = isset($_POST['key_id']) ? absint($_POST['key_id']) : 0;
                $description = sanitize_text_field(wp_unslash($_POST['description']));
                $permissions = (in_array(wp_unslash($_POST['permissions']), array( 'read', 'write', 'read_write' ), true)) ? sanitize_text_field(wp_unslash($_POST['permissions'])) : 'read';
                $user_id     = absint($_POST['user']);
    
                // Check if current user can edit other users.
                if ($user_id && ! current_user_can('edit_user', $user_id)) {
                    if (get_current_user_id() !== $user_id) {
                        throw new Exception(__('You do not have permission to assign API Keys to the selected user.', 'eys'));
                    }
                }
    
                if (0 < $key_id) {
                    $data = array(
                        'user_id'     => $user_id,
                        'description' => $description,
                        'permissions' => $permissions,
                    );
    
                    $wpdb->update(
                        $wpdb->prefix . 'woocommerce_api_keys',
                        $data,
                        array( 'key_id' => $key_id ),
                        array(
                            '%d',
                            '%s',
                            '%s',
                        ),
                        array( '%d' )
                    );
                    $response                    = $data;
                    $response['consumer_key']    = '';
                    $response['consumer_secret'] = '';
                    $response['message']         = __('API Key updated successfully.', 'eys');
                } else {
                    $consumer_key    = 'ck_' . wc_rand_hash();
                    $consumer_secret = 'cs_' . wc_rand_hash();
                    $data = array(
                        'user_id'         => $user_id,
                        'description'     => $description,
                        'permissions'     => $permissions,
                        'consumer_key'    => wc_api_hash($consumer_key),
                        'consumer_secret' => $consumer_secret,
                        'truncated_key'   => substr($consumer_key, -7),
                    );
    
                    $wpdb->insert(
                        $wpdb->prefix . 'woocommerce_api_keys',
                        $data,
                        array(
                            '%d',
                            '%s',
                            '%s',
                            '%s',
                            '%s',
                            '%s',
                        )
                    );
    
                    $key_id                      = $wpdb->insert_id;
                    $response                    = $data;
                    $response['consumer_key']    = $consumer_key;
                    $response['consumer_secret'] = $consumer_secret;
                    update_option('eys_woocommerce_api_consumer_key', $consumer_key);
                    update_option('eys_woocommerce_api_consumer_secret', $consumer_secret);
                    $query ="signup?wc=1&ck=".  $consumer_key."&cs=". $consumer_secret."&url=".get_site_url();
                    $ExportYourStore_url =  $ExportYourStore_url.$query;
                }
            } catch (Exception $e) {
                echo $e->getMessage();
            }
        }
        
        include_once(plugin_dir_path(__FILE__) .'/inc/eys_button_html.php');
    }

        
    /**
     * create_rest_api_redirect_eys_user
     *
     * @return void
     */
    public function create_rest_api_redirect_eys_user()
    {
        register_rest_route(
            'eysApi/v1',
            '/redirect',
            array(
            'methods' => 'POST',
            'callback' =>  array($this,'response_rest_api_redirect_eys_user'),
            'args' => array(
                'user_id' => array(
                  'validate_callback' => 'is_numeric'
                ),
              ),
            'permission_callback' => '__return_true'
        )
        );
    }
    
    /**
     * response_rest_api_redirect_eys_user
     *
     * @param  mixed $request
     * @return void
     */
    public function response_rest_api_redirect_eys_user($request)
    {
        $response = array();
        $parameters = $request->get_params();
        $response['parameters'] = $parameters;
        if (get_option('eys_woocommerce_api_consumer_key') && get_option('eys_woocommerce_api_consumer_secret')) {
            $consumer_key = get_option('eys_woocommerce_api_consumer_key');
            $consumer_secret = get_option('eys_woocommerce_api_consumer_secret');
            if ((isset($parameters['key']) && $parameters['key'] ==  $consumer_key) &&  (isset($parameters['secret']) && $parameters['secret'] ==  $consumer_secret)) {
                if (isset($parameters['user_id'])) {
                    update_option('eys_exportyourstore_returned_user_id', $parameters['user_id']);
                    if (!empty($parameters['user_id'])) {
                        $response['code'] = 200;
                        $response['message'] = __("ExportYourStore user ID is saved successfully", "eys");
                        return new WP_REST_Response($response, 123);
                    } else {
                        $response['code'] = 403;
                        $response['message'] = __("ExportYourStore User id is not sent with redirect url", "eys");
                    }
                } else {
                    $response['code'] = 403;
                    $response['message'] = __("ExportYourStore User id is not sent with redirect url ", "eys");
                }
            } else {
                $response['code'] = 403;
                $response['message'] = __("ExportYourStore sent parameters key and secret does not match with woocommerce rest api keys. ", "eys");
            }
        } else {
            $response['code'] = 403;
            $response['message'] = __("No woocommerce Rest api keys are found.", "eys");
        }
        return $response;
    }

    
    /**
     * eys_is_wc_active
     *
     * @return void
     */
    private function eys_is_wc_active()
    {
        if (!function_exists('is_plugin_active')) {
            require_once ABSPATH . '/wp-admin/includes/plugin.php';
        }
        if (is_plugin_active('woocommerce/woocommerce.php')) {
            $is_active = true;
        } else {
            $is_active = false;
        }

        // Do the WC active check
        if (false === $is_active) {
            add_action('admin_notices', array($this, 'eys_notice_activate_wc'));
        }
        return $is_active;
    }

    /**
    * Display WC active notice
    *
    * @access public
    * @since  1.0.0
    */
    public function eys_notice_activate_wc()
    {
        echo '<div class="error">';
        echo '<p>';
        printf(__('Please install and activate %sWooCommerce%s for ExportYourStore', 'eys'), '<a href="' .esc_url(admin_url('plugin-install.php?tab=search&s=WooCommerce&plugin-search-input=Search+Plugins')). '">', '</a>');
        echo '</p>';
        echo '</div>';
    }

    /**
     * WOOCOMMERCE_VERSION admin notice
     *
     * @since 0.1.0
     */
    public function eys_admin_error_notice()
    {
        $message = __(' ExportYourStore requires WooCommerce 3.0 or newer', 'eys');
        echo "<div class='error'><p>".esc_html($message)."</p></div>";
    }
}

$pbvwp = new eys();
